
import { Component, OnInit, OnDestroy, ViewChild, ElementRef, ChangeDetectorRef } from '@angular/core';


import { ProbeService, requesterID } from '../../../commons/service/probe.service';

import { Zone, Host, Port, Service, DiscoverHost } from '@overflow/model/discovery';
import { toMetaIPType, MetaIPTypeEnum } from '@overflow/model/meta';

import { RPCSubscriber } from '@overflow/commons/ui/decorator/RPCSubscriber';
import { DiscoveryConfigService } from '../../../commons/service/discovery-config.service';
import { Observable } from 'rxjs';

import * as d3 from 'd3';
import { Node } from '../../../commons/model/node';
import { Link } from '../../../commons/model/link';

@Component({
  selector: 'app-pages-home',
  templateUrl: './home-page.component.html',
  styleUrls: ['./home-page.component.scss'],
})
export class HomePageComponent implements OnInit, OnDestroy {

  private discoveryTargetRef: ElementRef;

  zone: Zone;
  discoverHost: DiscoverHost;

  @ViewChild('discoveryTarget') set discoveryTarget(content: ElementRef) {
    this.discoveryTargetRef = content;
  }

  blockedPanel = false;
  displaySidebar = false;

  blockedDocument = false;

  showIntro = true;
  selectedNode = null;

  // private visNetwork: any;
  // private nodeSet: any;
  // private edgeSet: any;

  private nodes: Node[];
  private links: Link[];
  public simulation: d3.Simulation<Node, Link>;

  constructor(
    private changeDetector: ChangeDetectorRef,
    private probeService: ProbeService,
    private discoveryConfigService: DiscoveryConfigService,
  ) {
    this.nodes = [];
    this.links = [];
  }

  ngOnInit() {
    this.probeService.subscribeNotification(this);
    // this.startDiscovery("");
    this.discoveryConfigService.zone.subscribe(res => {
      this.zone = res as Zone;
    });
    this.discoveryConfigService.discoverHost.subscribe(res => {
      this.discoverHost = res as DiscoverHost;
    });
    this.discoveryConfigService.started.subscribe(res => {
      if (res) {
        this.startDiscovery();
      }
    });
  }

  ngOnDestroy(): void {
    this.probeService.unsubscribeNotification(this);
  }

  blockDocument() {
    this.blockedDocument = true;
    setTimeout(() => {
      this.blockedDocument = false;
    }, 3000);
  }

  startDiscovery() {
    console.log('########################################################');
    console.log(this.zone);
    console.log(this.discoverHost);
    console.log('########################################################');
    this.showIntro = false;
    this.changeDetector.detectChanges();

    const width = this.discoveryTargetRef.nativeElement.clientWidth;
    const height = this.discoveryTargetRef.nativeElement.clientHeight;

    this.simulation = d3
      .forceSimulation<Node, Link>()
      .nodes(this.nodes)
      .force('charge', d3.forceManyBody().strength(-120))
      .force('link', d3.forceLink(this.links).distance(150))
      .force('center', d3.forceCenter(width / 2, height / 2))
      .force('collision', d3.forceCollide().radius(function (node: Node) {
        return node.r;
      }))
      ;

    this.nodes.push(
      { id: '192.168.1.0/24', fx: width / 2, fy: height / 2 },
      // { id: '192.168.1.1' },
      // { id: '192.168.1.2' },
    );

    this.links.push(
      // { source: this.getNode('192.168.1.0/24'), target: this.getNode('192.168.1.1') },
      // { source: this.getNode('192.168.1.0/24'), target: this.getNode('192.168.1.2') }
    );

    this.simulationRestart();

    const zone: Zone = {
      network: '192.168.1.0/24',
      iface: 'enp3s0',
      metaIPType: toMetaIPType(MetaIPTypeEnum.V4),
      address: '192.168.1.101',
      mac: '44:8a:5b:f1:f1:f3',
    };

    const discoverHost: DiscoverHost = {
      metaIPType: toMetaIPType(MetaIPTypeEnum.V4),
      firstScanRange: '192.168.1.1',
      lastScanRange: '192.168.1.254',
      discoveryConfig: {
      },
      discoverPort: {
        firstScanRange: 1,
        lastScanRange: 65535,
        includeTCP: true,
        includeUDP: true,
        discoverService: {
        }
      }
    };

    this.probeService.send('DiscoveryService.DiscoverHost', requesterID, zone, discoverHost);

    // this.nodeSet = new vis.DataSet([
    //   {
    //     id: '192.168.1.0/24', label: 'Zone', group: 'zone',
    //     shape: 'image',
    //     image: '../../assets/Windows_logo.png',
    //   },
    // ]);

    // this.edgeSet = new vis.DataSet([]);

    // const options = {
    //   width: '100%',
    //   height: '100%',
    //   groups: {
    //     zone: { color: { background: 'red' }, borderWidth: 5 },
    //     host: { color: { background: 'blue' }, borderWidth: 3 },
    //     service: { color: { background: 'white' }, borderWidth: 1 },
    //   }
    // };

    // this.visNetwork = new vis.Network(this.discoveryTargetRef.nativeElement, { nodes: this.nodeSet, edges: this.edgeSet }, options);
    // this.visNetwork.on('click', (params) => {
    //   const ids = params.nodes;
    //   const clickedNode = this.nodeSet.get(ids);
    //   if (clickedNode.length > 0) {
    //     this.displaySidebar = true;
    //     this.selectedNode = clickedNode[0];
    //   } else {
    //     this.displaySidebar = false;
    //     this.selectedNode = null;
    //   }
    // });

    // this.probeService.send(
    //   'DiscoveryService.DiscoverHost',
    //   requesterID,
    //   this.discoveryConfigService.getZone(),
    //   this.discoveryConfigService.getDiscoverHost()
    // );
  }

  simulationRestart() {
    // Update and restart the simulation.
    this.simulation
      .nodes(this.nodes)
      .force('link', d3.forceLink(this.links).distance(150))
      .alpha(1)
      .restart()
      ;
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.Queueing' })
  public DiscoveryQueueing(queueingDate: Date) {
    console.log('DiscoveryQueueing', queueingDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingFailed' })
  public DiscoveryQueueingFailed(queueingFailedDate: Date) {
    console.log('DiscoveryQueueingFailed', queueingFailedDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingTimeout' })
  public DiscoveryQueueingTimeout(queueingTimeoutDate: Date) {
    console.log('DiscoveryQueueingTimeout', queueingTimeoutDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStart' })
  public DiscoveryStart(startDate: Date) {
    console.log('DiscoveryStart', startDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStop' })
  public DiscoveryStop(stopDate: Date) {
    console.log('DiscoveryStop', stopDate);

    this.simulationRestart();

    this.simulation.stop();
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryError' })
  public DiscoveryError(err: Error) {
    console.log('DiscoveryError', err);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredHost' })
  public DiscoveredHost(host: Host) {
    console.log('DiscoveredHost', host);
    const hostId = `${host.address}`;
    const zoneNode = this.getNode('192.168.1.0/24');
    const hostNode = new Node(hostId);
    hostNode.x = zoneNode.x;
    hostNode.y = zoneNode.y;

    this.nodes.push(hostNode);
    this.links.push(new Link(zoneNode, hostNode));
    this.simulationRestart();
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredPort' })
  public DiscoveredPort(port: Port) {
    console.log('DiscoveredPort', port);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredService' })
  public DiscoveredService(service: Service) {
    console.log('DiscoveredService', service);
    const hostId = service.port.host.address;
    const serviceId = `${service.port.host.address}-${service.port.portNumber}-${service.key}`;

    const hostNode = this.getNode(hostId);
    const serviceNode = new Node(serviceId);
    serviceNode.x = hostNode.x;
    serviceNode.y = hostNode.y;

    this.nodes.push(serviceNode);
    this.links.push(new Link(hostNode, serviceNode));

    this.simulationRestart();
  }

  private getNode(id: string): Node {
    let _n: Node;
    this.nodes.some((node): boolean => {
      if (node.id === id) {
        _n = node;
        return true;
      }
      return false;
    });
    return _n;
  }
}
